/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.web.file;

import java.util.HashMap;
import java.util.Map;

import dk.vajhoej.vms.web.FunctionModule;
import dk.vajhoej.vms.web.Result;
import dk.vajhoej.vms.web.Util;

/**
 * Class FileModule serve files from directory tree.
 */
public class FileModule implements FunctionModule {
	private static class FileType {
		private String extension;
		private boolean binary;
		private String mimeType;
		public FileType(String extension, boolean binary, String mimeType) {
			this.extension = extension;
			this.binary = binary;
			this.mimeType = mimeType;
		}
		@SuppressWarnings("unused")
		public String getExtension() {
			return extension;
		}
		public boolean isBinary() {
			return binary;
		}
		public String getMimeType() {
			return mimeType;
		}
	}
	private static Map<String,FileType> cap = new HashMap<String,FileType>();
	static {
		cap.put("txt", new FileType("txt", false, "text/plain"));
		cap.put("html", new FileType("html", false, "text/html"));
		cap.put("css", new FileType("css", false, "text/css"));
		cap.put("js", new FileType("js", false, "text/javascript"));
		cap.put("gif", new FileType("gif", true, "image/gif"));
		cap.put("jpg", new FileType("jpg", true, "image/jpeg"));
		cap.put("jpeg", new FileType("jpeg", true, "image/jpeg"));
		cap.put("png", new FileType("png", true, "image/png"));
		cap.put("ico", new FileType("ico", true, "image/x-icon"));
		cap.put("zip", new FileType("zip", true, "application/zip"));
		cap.put("exe", new FileType("exe", true, "application/octet-stream"));
		cap.put("jar", new FileType("jar", true, "application/java-archive"));
	}
	private String dir;
	/**
	 * Constrict instance.
	 * @param dir directory to serve
	 */
	public FileModule(String dir) {
		this.dir = dir;
	}
	public String getContext() {
		return ""; // file is a good default context
	}
	public Result getResponse(String remurl, String query) {
		try {
			if(remurl.endsWith("/")) remurl = remurl + "index.html";
			if(!Util.isValidFile(dir, remurl)) return new Result(false, "Invalid file specification: " + dir + remurl, null, null);
			String fnm = dir + remurl;
			String[] parts = fnm.split("\\.");
			String ext = parts[parts.length - 1];
			FileType ftyp = cap.get(ext);
			if(ftyp != null) {
				if(ftyp.isBinary()) {
					return new Result(true, null, ftyp.getMimeType(), Util.loadBinary(fnm));
				} else {
					return new Result(true, null, ftyp.getMimeType(), Util.loadBinary(fnm)); // avoid many char set issues
				}
			} else {
				return new Result(false, "Unknown file type: " + ext, null, null);
			}
		} catch (Exception e) {
			return new Result(false, "Exception: " + e.getMessage(), null, null);
		}
	}
}
